/* *****************************************************************************
Copyright 2011 Leonardo Uieda

Tesseroids is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Tesseroids is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Tesseroids.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************************** */

/** \file
Define constants used, like the gravitational constant and unit conversions.

<b>All values are in SI units!</b>

@author Leonardo Uieda
@date 24 Jan 2011
*/

#include "constants.h"

/** \var MEAN_EARTH_RADIUS
Mean Earth radius [\f$ m \f$] */
const double MEAN_EARTH_RADIUS = 6378137.0;

/** \var G
The gravitational constant [\f$ m^3*kg^{-1}*s^{-1} \f$] */
const double G = 0.00000000006673;

/** \var SI2EOTVOS
Conversion factor from SI units to Eotvos
[\f$ \frac{1}{s^2} = 10^9\ Eotvos \f$] */
const double SI2EOTVOS = 1000000000.0;

/** \var SI2MGAL
Conversion factor from SI units to mGal
[\f$ 1 \frac{m}{s^2} = 10^5\ mGal \f$] */
const double SI2MGAL = 100000.0;

/** \var PI
Pi */
const double PI = 3.1415926535897932384626433832795;

/** \var TESSEROID_SIZE_RATIO
Minimum distance-to-size ratio for computations to be accurate */
const int TESSEROID_SIZE_RATIO = 4;
